#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "ABaCA.hh"
#include "Data.hh"
#include "Functions.h"



//	El constructor de esta clase recibira como parametro, la
//	referencia  de nombre de archivo, con diferentes extensiones 
//	se obtencran todos los datos requeridos.
Data::Data(char *fdat, char *fpars)
{	

	D_datafile = strdup(fdat);
	D_parsfile = strdup(fpars);

        printf("\nLeyendo el proyecto: %s\n", D_datafile);
        
	FILE *Da;
	char  nam[LEN];

	//	Se le agrega la extencin al archivo de datos.
	sprintf( nam, "%s.pars", D_parsfile);

	if ((Da = fopen( nam , "r" )) == NULL)
	{	
		printf("No se pudo abrir archivo %s con los parametros\n", nam);
		exit(0);
	}
	       
      	if (fscanf( Da, " MAXK: %d MAXM: %d", &D_Maxk, &D_Maxm) != 2)
        {
		printf("Problemas al leer MAXK, o MAXM\n");
		exit(0);
	}
        
        D_Maxk += 2; //se le agrega dos por seguridad a k=0.
        D_Maxm += 2;
        if (D_Initial() == 0)
        {
        	printf("Problemas al abrir memoria para los datos!!!!!\n\n");
                
                exit(0);
        }

	double ax1, ax2;
      	if (fscanf( Da, " LRUN: %lf ITER: %lf", &ax1, &ax2) != 2)
        {
		printf("Problemas al leer LRUN o ITER\n");
		exit(0);
	}
	D_longrun = (int) ax1;
        D_iter = (int) ax2;
        
      	if (fscanf( Da, " PART: %lf", &D_part) != 1)
        {
		printf("Problemas al leer PART\n");
		exit(0);
	}

	int a,b;
      	if (fscanf( Da, " alfa: %d beta: %d d: %d", &a, &b, &D_d) != 3)
        {
		printf("Problemas al leer alfa, beta o d\n");
		exit(0);
	}

	alpha = (double) a;
        beta = (double) b;

        if ((D_d > 0) && (fscanf( Da, " Stop: %d", &stop) == 1))
        {
        	//valores para tiempos de paro
        	if (fscanf( Da, " Maxjump: %d a: %lf b: %lf",
        		&D_maxjump, &D_a, &D_b) != 3)
       		{
				printf("Problemas al leer D, a o b\n");
				exit(0);
		}
         	if (fscanf( Da, " dec: %d maxj: %d",
         		&D_dec, &D_maxj) != 2)
       		{
				printf("Problemas al leer dec o maxj\n");
				exit(0);
		}
	}
       	else
       		stop=0;
	

	fclose(Da);

	printf("MAXK: %d, MAXM: %d .\n",
        	D_Maxk-2, D_Maxm-2);
	printf("LRUN: %d ITER: %d\n", D_longrun, D_iter);
	printf("PART: %f\n", D_part);
	printf("alfa: %d beta: %d d: %d\n\n", a, b, D_d);
	
        if (stop)
        {
        	printf("\nValores para tiempos de paro:\n");
        	printf("Maxjump: %d a: %lf b: %lf\n",
        		D_maxjump, D_a, D_b);
         	printf("dec: %d maxj: %d\n\n",
         		D_dec, D_maxj);
	}
	        	
        
	//	Se le agrega la extencin al archivo de datos.
	sprintf( nam, "%s.dat", D_datafile);

	//	Se intenta abrir el archivo de datos para leerlos.
	if( (Da = fopen( nam , "r")) == NULL)
	{	
		printf("No se pudo abrir archivo %s con los datos\n", nam);
		exit(0);
	}
	
	int i = 0, unit, valor;
	//Se leen los datos del archivo.
	while (!feof(Da))
	{
		fscanf( Da, " %d %d\n", &unit, &valor);

                	
		D_j[i] = unit;
		D_X[i] = valor;
		
		if(i > 0)
		{			
			D_h[i] = (D_j[i] - D_j[i-1]);

                        
                        if(i == 1)
			{
				minp = maxp = D_h[i];
			}
			else
			{
				minp = min(minp, D_h[i]);
				maxp = max(maxp, D_h[i]);
			}

		}
		i++;
                if (i>=D_Maxm)
                {
                	printf("Existen mas registros en %s que %d!!\n",
                        	nam, D_Maxm);
                        
                        exit(0);
                }
	}
	printf("minh= %d, maxh= %d\n", minp, maxp);

	//	Se cierra el archivo.
	fclose(Da);
	
	//	Se guarda la cantidad de datos contenidos en el archivo.
	D_m = i-1;
        
        //Se calcula u, la suma de lo observado.  U() incluyen los predichos.
	int S = 0;
	for(int i = 1; i<= D_m; i++)
        	S += D_X[i];
        u = S;
	
	
	printf("\nEl total de registros observados son %d.\n", D_m);


	
	//	Se inicializan los datos de los valores futuros,
	//	si los hay, a cero.
        
        
        if (D_m + D_d >= D_Maxm)
        {
        	printf("Con los datos futuros se exede %d!!\n",
              		D_Maxm);
                        
               exit(0);
        }

	for(int i = D_m+1; i<= D_m + D_d; i++)
        {
        	D_X[i] = 0;
                D_j[i] = D_j[D_m] + (i-D_m);
                D_h[i] = 1;  //En periodos consecutivos.
        }
        minp = 1;
                
	printf("Se tienen %d valores para datos futuros.\n", D_d);

	D_n = D_j[D_m + D_d];  //El total de periodos.

	

	//se lee la inicial para k
	sprintf( nam, "%s.Priork", D_parsfile);
		
	if( (Da = fopen( nam, "r")) == NULL)
	{
		printf("No existe el archivo para la inicial de k %s.\n", nam);
	}
	else
        {
		double val;
                int num;
		//	Se leen los datos del archivo.
		while( !feof ( Da ) )
		{
			fscanf( Da, " %d %lf\n", &num, &val);
                	if (num>=D_Maxk)
                	{
                		printf("k excede %d en %s!!\n",
                        		 D_Maxk, nam);
                        
                        	exit(0);
                	}
			D_prk[num] = val;
		}
		D_Maxk = num+1;
		printf("Inicial para k leida de %s .\n", nam);
		//	Se cierra el archivo.
		fclose(Da);
        }


        
        D_P0 = alpha/(alpha+beta); //Betasim( alpha, beta); //valor inicial para p.

	D_k0 = U(); //valor inicial para k.
        
        printf("k0= %d, p0= %f\n", D_k0, D_P0);
        
        
}


//Abrimos todos los arreglos y los inicializamos
int Data::D_Initial()
{
	int k=1;

	D_X = new int[D_Maxm];
        if (D_X == NULL)
        	k=0;

        D_h = new int[D_Maxm];
        if (D_h == NULL)
        	k=0;

        D_j = new int[D_Maxm];
        if (D_j == NULL)
        	k=0;

        D_prk = new double[D_Maxk];
        if (D_prk == NULL)
        	k=0;


	if (k==0)  //si hubo problemas.
        	return k;

	for(int i = 0; i< D_Maxm; i++)
	{
		D_X[i] = 0;
		D_h[i] = 0;
		D_j[i] = 0;
	}

	for(int i = 0; i< D_Maxk; i++)
		D_prk[i] = 1.0;

	minp = maxp = 0;

	return k;
}










void Data::PrintCum()
{
	int S = 0;
        printf("%3s: %3s  %3s  %3s  %3s\n", "l", "j_l", "h_l", "X_l", "u_l");
	for(int i = 0; i<= D_m + D_d; i++)
		printf("%3d: %3d  %3d  %3d  %3d \n",
                	i, D_j[i], D_h[i], D_X[i], (S += D_X[i]));
}

char *Data::Fnam()
{
	strcpy( ot, D_datafile);
        
        return ot;
}

int Data::U()
{
	int S = u;
	for(int i = D_m+1; i<= D_m + D_d; i++)
        	S += D_X[i];
        
        return S;
}

int Data::V()
{
	int S = -1*D_n*U();
        
	for(int i = 1; i<= D_m + D_d; i++)
        	S += D_j[i-1]*D_X[i];
        return S;
}
        
