// Principal Program.
#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "Functions.h"
#include "ABaCA.hh"

MCMC::MCMC(Data *Da):Cumm(Da)
{
	p=D->P0(); //valores iniciales
	k=D->K0();
        
	ax = new double[D->Maxk()];
	if (ax == NULL)
	{
		printf("Insuficiente memoria, MCMC::ax!!!\n");
		exit(0);
	}

	ax1 = new int[D->Maxk()];
	ax2 = new int[D->Maxk()];       
	if ((ax1 == NULL) || (ax2 == NULL))
	{
		printf("Insuficiente memoria, MCMC::ax1,ax2!!!\n");
		exit(0);
	}
        
        for (int k=0; k<D->Maxk(); k++)
        {
        	ax1[k] = 0;
                ax2[k] = 0;
        }
}


#define PAZ 10000

void MCMC::DoSample()
{
	printf("\nEmpezamos con el MCMC\n");
        
        //D->PrintCum();
        
        int it=1;
        for (; it<=D->Long(); it++)
        {
        	//en Pass() se actualiza k, p y las observaciones futuras.
        	Pass();
                
                //pacificador
                if ((it % PAZ) == 0)
                	printf("It: %7d\n", it);
        }
        
        FILE *out;
        char nam[255];
        sprintf( nam, "%s.sim", D->Fnam());
        if ((out=fopen( nam, "w")) == NULL)
        {
        	printf("No se pudo abrir el archivo %s para guardar las simulaciones\n", nam);
                exit(0);
	}
        
        int s;      
        for (; it<=D->Iter(); it++)
        {
        	Pass();
                
                //Escribimos las simulaciones en out
                fprintf( out, "%3d  %8.6f\t", k, p);
                ax1[k]++;
                //Las futuras.
                s=0;
                for (int d=1; d<=D->Fut(); d++)
                {
                	s += D->GetX(d);
                	fprintf( out, "%3d  ", D->GetX(d));
                }
                fprintf( out, "\t%4d\n", s);
                ax2[s]++;

                if ((it % PAZ) == 0)
                	printf("It: %7d\n", it);
                
        }
        
        fclose(out);
        
}

/*Este metodo toma el archivo D->Fnam().sim y analiza las simulaciones.
Crea una post. para k y para p y una post. para la suma de las observaciones
futuras...Puede hacer mas cosas despues, como tiempos de paro etc.*/
void MCMC::MakePosts()
{
	printf("\nAnalizando las simulaciones en %s.sim .\n", D->Fnam());

        FILE *out;
        char nam[255];
        sprintf( nam, "%s.Postk", D->Fnam());
        if ((out=fopen( nam, "w")) == NULL)
        {
        	printf("No se pudo abrir %s para la posterior  de k.\n", nam);
                exit(0);
	}
        

        printf("Escribiendo la posterior aproximada de k en %s .\n", nam);
        int ssize = D->Iter()-D->Long();

        for (k=D->uu()-3; k<D->uu(); k++) //agregamos primero unos ceros antes de u         	
        	fprintf( out, "%3d\t\t%g\n", k, 0.0);
	double cum=0.0; //100*CP% de la probabilidad
        for (	k=D->uu();
        	(k < D->Maxk()) && ((cum += ax1[k-1]/(double) ssize) < CP);
        	k++)
         	fprintf( out, "%3d\t\t%g\n", k, ax1[k]/(double) ssize);
        
        fclose(out);
        
        if (D->Fut() > 0)
        {
        	sprintf( nam, "%s.PostS", D->Fnam());
        	if ((out=fopen( nam, "w")) == NULL)
        	{
        		printf("No se pudo abrir %s para la posterior de S.\n", nam);
        	        exit(0);
		}
        
        	printf("Escribiendo la posterior aproximada de S en %s .\n", nam);
        	printf("(Numero de especies nuevas a ser vistas en %d periodos adicionales.)\n",
       		 	D->Fut());  
	 	cum=0.0; //100*CP% de la probabilidad
        	for (k=0;
        	    (k < D->Maxk()) && ((cum += ax2[k-1]/(double) ssize) < CP);
        	    k++)
        		fprintf( out, "%3d\t\t%g\n", k, ax2[k]/(double) ssize);
        
        	fclose(out);
       }
        
       printf("No se calcula la posterior para p, vea %s.sim .\n", D->Fnam());
                
}




