#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "Functions.h"

/*Definimos esto si queremos el procedimiento como en el articulo, sin adaptar 
la envoltura. Berger y Sun, JASA 1993, V.88 p.424 
#define BERGER */

#define PINIT 0.3
#define HBK(k)  h[k] = logf( p[k], pt); \
		b[k] = h[k] - p[k]*hd[k];

#define UK(k) 	u[k] = (b[k] - b[k-1])/(hd[k-1] - hd[k]);

#define CHG(k)	p[k] = P; \
		h[k] = hax; \
		hd[k] = hdax; \
		b[k] = h[k] - p[k]*hd[k];


/* a = alpha, b = nk + v + beta */
struct pars
{
	double a;
	double b;
	int m;
	const int *x;
	const int *hh;
};

double logf( double p, const struct pars *pt)
{
	int l;
	double S=0.0;
	
	for (l=0; l<pt->m; l++)
		S += pt->x[l]*log(1.0 - pow( 1.0-p, (double) pt->hh[l]));
	
	return ( (pt->a - 1.0)*log(p) + (pt->b - 1.0)*log(1.0-p) + S );
}

double hdif(double p, const struct pars *pt)
{
	int l;
	double S=0.0, a, b, c;
	
	for (l=0; l<pt->m; l++)
        {
        
		S += (double) (pt->x[l]*pt->hh[l]) *
( pow( 1.0-p, (double) pt->hh[l]-1.0) / (1.0 - pow( 1.0-p, (double) pt->hh[l])) );
	}

	return ( (pt->a - 1.0)/p - (pt->b - 1.0)/(1.0-p) + S );
}
	

int simlogf(double *rtp, const struct pars *pt)
{
	double p[3];
	double h[3], hd[3];
	double b[3];
	double u[4];
	double V[3], SV=0.0, Cmm, C;
	double U, P, hax, hdax, hu;
	int k, i, cnt=0;
	

/*encuentra p[0] y p[2]*/
	
	p[2] = PINIT;
	hd[2] = hdif( p[2], pt);
/* en este caso PINIT es muy grande! */
	if (hd[2] >= 0.0)
	{
		*rtp = -1.0;
                printf("simp: PINIT muy grande!!\n");
		return -1;
	}
	i=0;
	while (1)
	{
		i++;
                if (i == 2000)
		{
			*rtp = -1.0;
                	printf("simp: 1: Mas de 2000 iteraciones!!\n");
			return -1;
		}
                
		p[0] =  p[2]*Un01(); /*alternativo: p[0] = drand48()*p[2] */ 
		hd[0] = hdif( p[0], pt);
/* Verificamos si esto es menor que 709, el numero maximo para el cual se
puede calcular exp(709).  Esto es para poder calcular la P abajo y que no
salga Inf */
		if (p[2]*hd[0] > 709.0)
			continue;
		else
		{
			if (hd[0] > 0.0)
				break;
			else
			{
				p[2] = p[0];
				hd[2] = hd[0];
			}
		}
	/*siempre sabemos que hd[2] < 0.0 y al final tenemos hd[0]>0 y hd[2]<0*/
	}
/*calculamos hk's y b'ks y p1*/
	HBK(0);
	HBK(2);
		
	p[1] = (b[2] - b[0])/(hd[0] - hd[2]);
	hd[1] = hdif( p[1], pt);
	HBK(1);


		
/*contamos a u igual que en el articulo, u[0]=0 y u[3]=PINIT*/
	u[0]=0;
	u[3]=PINIT;


	UK(1);
	UK(2);

/*usamos C para cambiar la escala y poder hacer los calculos, tomando el
promedio de los exponentes*/
	C=0.0;
	for (k=0; k<3; k++)
		C += (u[k+1]*hd[k]+b[k]);
	C /= 3.0;
	for (k=0; k<3; k++)
	{
		V[k] = (exp(u[k+1]*hd[k]+b[k]-C)-exp(u[k]*hd[k]+b[k]-C))/hd[k];
		SV += V[k];
	}

#ifndef BERGER

	while (1)
	{
		if (++cnt > 2000)
		{
			*rtp = -1;
                	printf("simp: 2: Mas de 2000 iteraciones!!\n");
			return cnt-1;
		}

/* simulamos la k para la exponencial y despues la P*/
		U = Un01();
		Cmm = 0.0;
		k=0;
		while (U > (Cmm += V[k]/SV))
			k++;
		U = Un01();
		P = log(U*exp(u[k+1]*hd[k]) + (1-U)*exp(u[k]*hd[k]))/hd[k];	


/*ahora vemos si esta dentro de la curva*/
		U = Un01();
		hax = logf( P, pt);
		hu = P*hd[k] + b[k];
		if (U <= exp(hax - hu))
		{
			*rtp = P;
			return cnt;
		}

/* No esta, ahora adaptamos la envoltura con la nueva P */

/*Si esta entre p0 y p2 y la pendiente en h(P) es menor que la del
punto intermedio, P lo hacemos p1 */
		if (fabs((hdax = hdif( P, pt))) < fabs(hd[1]))
		{
			if ((P > p[0]) && (P < p[2]))
			{
				CHG(1);
				UK(1);
				UK(2);

				/*Actualizamos los Vk's y SV */
				SV = 0.0;
				C=0.0;
				for (k=0; k<3; k++)
					C += (u[k+1]*hd[k]+b[k]);
				C /= 3.0;
				for (k=0; k<3; k++)
				{
					V[k] = (exp(u[k+1]*hd[k]+b[k]-C)-exp(u[k]*hd[k]+b[k]-C))/hd[k];
					SV += V[k];
				}
				continue;
			}
		}

/*Sino, lo hacemos p[0] si P es menor que p[1] y p[2] en el otro caso. */
		if (P < p[1])
		{
			CHG(0);
			UK(1);
		}
		else
		{
			CHG(2)
			UK(2);
		}

		/*Actualizamos los Vk's y SV */
		SV = 0.0;
		C=0.0;
		for (k=0; k<3; k++)
			C += (u[k+1]*hd[k]+b[k]);
		C /= 3.0;
		for (k=0; k<3; k++)
		{
			V[k] = (exp(u[k+1]*hd[k]+b[k]-C)-exp(u[k]*hd[k]+b[k]-C))/hd[k];
			SV += V[k];
		}
	}
#endif

#ifdef BERGER
	while (1)
	{
		if (++cnt > 2000)
		{
			*rtp = -1;
                	printf("simp: 3: Mas de 2000 iteraciones!!\n");
			return cnt-1;
		}

/* simulamos la k para la exponencial y despues la P*/
		U = drand48();
		Cmm = 0.0;
		k=0;
		while (U > (Cmm += V[k]/SV))
			k++;
		U = drand48();
		P = log(U*exp(u[k+1]*hd[k]) + (1-U)*exp(u[k]*hd[k]))/hd[k];	


/*ahora vemos si esta dentro de la curva*/
		U = drand48();
		hax = logf( P, pt);
		hu = P*hd[k] + b[k];
		if (U <= exp(hax - hu))
		{
			*rtp = P;
			return cnt;
		}
/*Y aqui no adaptamos la envoltura */
	}

#endif

}

void UnESimp(double *rtp, int *cnt, double alpha, double b, int m,const int*x,const int *hh) {
	struct pars pr;
	
	pr.a = alpha;
	pr.b = b; /* a = alpha, b = nk + v + beta */
	pr.m = m;   
	pr.x = x;    /*arreglo conteniendo las x's*/
	pr.hh = hh;  /*arreglo conteniendo la h's*/
	
        /*se regresa el resultado en rtp, y en cnt el numero de iteraciones*/
	*cnt = simlogf( rtp, &pr );
}




